@@ -88,7 +88,7 @@ class Provider(BaseProvider):
 
         Default blob size is 1 Mb.
         """
-        blob = [self.generator.random.randrange(256) for o in range(length)]
+        blob = [self.generator.random.randrange(256) for _ in range(length)]
         return bytes(blob) if sys.version_info[0] >= 3 else bytearray(blob)
 
     def md5(self, raw_output=False):
@@ -176,7 +176,7 @@ class Provider(BaseProvider):
             required_tokens) <= length, "Required length is shorter than required characters"
 
         # Generate a first version of the password
-        chars = [self.generator.random.choice(choices) for x in range(length)]
+        chars = [self.generator.random.choice(choices) for _ in range(length)]
 
         # Pick some unique locations
         random_indexes = set()
